@@ -91,6 +91,12 @@ function(x, ...)
     x
 }
 
+Ops.simple_sparse_array <- 
+function(e1, e2) 
+{
+    stop("Not implemented.")
+}
+
 Summary.simple_sparse_array <-
 function(..., na.rm = FALSE)
 {
@@ -141,9 +147,13 @@ function(x, ...)
         return(x)
 
     nd <- length(x$dim)
+    pd <- prod(x$dim)
+
+    ## See now matrix.R for comment.
+    .protect <-  pd > slam_options("max_dense")
 
     ## Note there is a limit to representing integer numbers as 
-    ## doubles.
+    ## doubles (see above).
     spos <- function(i) {
 	if(!nrow(i)) 
 	    return(vector(mode = typeof(i), length = 0L))
@@ -162,12 +172,15 @@ function(x, ...)
         i <- ..1
         ## Single index subscripting.
         if(is.logical(i))
-            stop("Logical subscripting currently not implemented.")
+            stop("Logical vector subscripting currently not implemented.")
         else if(is.character(i))
-            stop("Character subscripting currently not implemented.")
+            stop("Character vector subscripting currently not implemented.")
         else if(!is.matrix(i)) {
-	    ## 52-bit safe
-	    if(prod(x$dim) > 4503599627370496)
+	    if (!is.numeric(unclass(i)))
+                stop(gettextf("Invalid subscript type: %s.",
+                              typeof(i)),
+                     domain = NA)
+	    if(log2(pd) > .Machine$double.digits)
 	      stop("Numeric vector subscripting disabled for this object.")
 	     ## Shortcut
 	     if(!length(i)) 
@@ -181,15 +194,15 @@ function(x, ...)
                 out <- vector(mode = typeof(x$v), length = length(i))
 		if(length(out)) {
 		    ## Missing values.
-		    is.na(i) <- i > prod(x$dim)
+		    is.na(i) <- i > pd
 		    is.na(out) <- is.na(i)
 		    i <- match(i, spos(x$i), 0L)
 		    out[i > 0L] <- x$v[i]
 		}
             } else if(!any(is.na(i)) && all(i <= 0)) {
-		if(prod(x$dim) > 16777216L)
+		if(.protect)
 		  stop("Negative vector subsripting disabled for this object.")
-                out <- vector(mode = typeof(x$v), prod(x$dim))
+                out <- vector(mode = typeof(x$v), pd)
                 out[spos(x$i)] <- x$v
 		## NOTE this fails if NAs are introduced by 
 		##	coercion to integer. 
@@ -202,17 +215,17 @@ function(x, ...)
 	     if(!nrow(i)) 
 		return(vector(mode = typeof(x$v), length = 0L))
 	     ## Ignore dimensions. 
-	     if(ncol(i) != nd) 
+	     if(ncol(i) != nd || !is.numeric(i)) 
 		return(do.call("[.simple_sparse_array", 
 			       list(x = x, as.vector(i))))
-            ## Note that negative values are not allowed in a matrix
-            ## subscript.
 	    if(is.double(i))
 		i <- trunc(i)
-            if(any(i < 0, na.rm = TRUE))
-                stop("Invalid subscript.")
 	    k <- .Call(R_all_row, i > 0, FALSE)
             i <- i[k, , drop = FALSE]
+            ## Note that negative values are not allowed in a matrix
+            ## subscript.
+            if(any(i < 0, na.rm = TRUE))
+                stop("Invalid subscript.")
             out <- vector(mode = typeof(x$v), length = nrow(i))
 	    if(length(out)) {
 		if(any(i > rep(x$dim, each = nrow(i)), na.rm = TRUE))
