@@ -5,7 +5,8 @@ function(x, MARGIN, INDEX, FUN, ...)
     UseMethod("rollup")
 
 rollup.array <-
-function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE) {
+function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE,
+	    MODE = "double") {
     if (is.character(MARGIN))
         MARGIN <- match(MARGIN, names(dimnames(x)))
     if (!all(match(MARGIN, seq_along(dim(x)), nomatch = 0L)))
@@ -29,11 +30,7 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE) {
 	z <- INDEX[[as.character(k)]]
 	z <-
 	if (is.null(z))
-	    structure(
-		rep(1L, d[k]),
-		levels = "1",
-		class  = "factor"
-	    )
+	    rep(factor(1L), d[k])
 	else {
 	    if (length(z) != d[k])
 		stop(gettextf("INDEX [%s] invalid length", k),
@@ -47,16 +44,17 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE) {
 	rm(z)
     }
     i <- .Call(R_vector_index, d, i)
-    i <- structure(
-	i,
-	levels = seq_len(prod(d)),
-	class  = "factor"
-    )
+    attributes(i) <-
+	list(levels = seq_len(prod(d)),
+	     class  = "factor")
     i <- split.default(x, i)
     names(i) <- NULL
     i <- lapply(i, FUN, ...)
-    if (all(unlist(lapply(i, length)) == 1L))
+    if (all(unlist(lapply(i, length)) == 1L)) {
 	i <- unlist(i, recursive = FALSE, use.names = FALSE)
+	if (is.null(i))
+	    i <- vector(MODE, 0L)
+    }
     ## NOTE see drop_simple_sparse_array
     if (DROP) {
 	if (any(d == 0L))
@@ -77,7 +75,7 @@ rollup.matrix <- rollup.array
 
 rollup.simple_sparse_array <-
 function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE,
-	    EXPAND = c("none", "sparse", "dense", "all")) {
+	    EXPAND = c("none", "sparse", "dense", "all"), MODE = "double") {
     if (is.character(MARGIN)) 
 	MARGIN <- match(MARGIN, names(dimnames(x)))
     if (!all(match(MARGIN, seq_along(dim(x)), nomatch = 0L)))
@@ -103,7 +101,7 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE,
 	    P <- array(1L, dim(I))
 	T <- vector("list", length(D))
 	for (k in seq_along(D)[-MARGIN])
-	    T[[k]] <- rep(1L, D[k])
+	    T[[k]] <- rep.int(1L, D[k])
     }
     N <- dimnames(x)
     if (is.null(N))
@@ -114,18 +112,20 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE,
 	stop("component 'v' contains 'ZERO' value(s)")
     for (k in MARGIN) {
 	z <- INDEX[[as.character(k)]]
-	z <-
-	if (is.null(z))
-	    structure(
-		rep(1L, D[k]),
-		levels = "1",
-		class  = "factor"
-	    )
-	else {
+	if (is.null(z)) {
+	    ## NOTE defer processing.
+	    if (EXPAND < 3L) {
+		if (EXPAND > 1L)
+		    T[[k]] <- D[k]
+		D[k] <- -1L
+		next
+	    }
+	    z <- rep(factor(1L), D[k])
+	} else {
 	    if (length(z) != D[k])
 		stop(gettextf("INDEX [%s] invalid length", k),
                      domain = NA)
-	    factor(z)
+	    z <- factor(z)
 	}
 	l <- levels(z)
 	D[k]   <- length(l)
@@ -166,15 +166,21 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE,
 	i <- match(i, k)
 	rm(k)
     } else {
+	if (EXPAND < 3L) {
+	    i <- which(D == -1L)
+	    if (length(i)) {
+		D[i]   <- 1L
+		N[i]   <- list("1")
+		I[, i] <- 1L
+	    }
+	}
 	i <- .Call(R_match_matrix, I, NULL, NULL)
 	I <- I[i[[2L]],, drop = FALSE]
 	i <-   i[[1L]]
     }
-    i <- structure(
-	i, 
-	levels = seq_len(dim(I)[1L]), 
-	class  = "factor"
-    )
+    attributes(i) <-
+	list(levels = seq_len(dim(I)[1L]),
+	     class  = "factor")
     if (EXPAND == 1L) {
 	V <- split.default(V, i)
 	rm(i)
@@ -213,6 +219,8 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE,
     }
     if (all(unlist(lapply(V, length)) == 1L)) {
 	V <- unlist(V, recursive = FALSE, use.names = FALSE)
+	if (is.null(V)) 
+	    V <- vector(MODE, 0L)
 	i <- V == vector(typeof(V), 1L)
 	i <- which(i)
 	if (length(i)) {
@@ -230,7 +238,7 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE,
 
 
 rollup.simple_triplet_matrix <- 
-function(x, MARGIN, INDEX = NULL, FUN = sum, ...) {
+function(x, MARGIN, INDEX = NULL, FUN = sum, ..., REDUCE = FALSE) {
     FUN <- match.fun(FUN)
     if (!identical(FUN, sum)) {
 	if (!is.null(list(...)$DROP))
@@ -260,11 +268,7 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ...) {
 		z <- INDEX[[as.character(k)]]
 		z <- 
 		if (is.null(z))
-		    structure(
-			rep(1L, dim(x)[k]),
-			levels = "1",
-			class  = "factor"
-		    )
+		    rep(factor(1L), dim(x)[k])
 		else {
 		    if (length(z) != dim(x)[k])
 			stop(gettextf("INDEX [%s] invalid length", k),
@@ -277,7 +281,8 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ...) {
 			  FALSE
 		      else
 			  as.logical(list(...)$na.rm), 
-		      FALSE
+		      as.logical(REDUCE),
+		      FALSE	## verbose
 		)
 	    }
 	)
@@ -287,10 +292,10 @@ function(x, MARGIN, INDEX = NULL, FUN = sum, ...) {
 
 ##
 rollup.default <-
-function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE) {
+function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE, MODE = "double") {
     if (!length(dim(x)))
 	stop("dim(x) must have a positive length")
-    rollup(as.array(x), MARGIN, INDEX, FUN, ..., DROP = DROP)
+    rollup(as.array(x), MARGIN, INDEX, FUN, ..., DROP = DROP, MODE = MODE)
 }
 
 ###
